# pySLEQP
This package provides a prototypical python implementation of a nonlinear programming
solver employing a sequential linear equality constrained quadratic programming (SLEQP)
method.

# License
pySLEQP is licensed under the terms of the pySLEQP academic license.
See file LICENSE.

# Installing Dependencies

pySLEQP has the following dependencies:

* gnu compiler toolchain (make, gfortran, gcc)
* BLAS
* LAPACK
* Python
* Numpy
* Scipy
* Cython
* GuRoBi   (free for academic use)
* METIS    (licensed with Apache License Version 2.0)
* HSL MA57 (free for academic use)
* GLTR     (free for academic use from GALAHAD Suite)

## Ubuntu/Debian Packages
To install the gnu compiler toolchain, BLAS, LAPACK, Python, Numpy, Scipy and Cython within Debian or Ubuntu,
you may execute:

    sudo apt-get install build-essential gfortran python-dev python-numpy python-scipy cython liblapack-dev libblas-dev

## GUROBI
You need to get GuRoBi at gurobi.com. Set the variable `GUROBIDIR` in the pysleqp
`Makefile` to the GuRoBi folder that containts include and lib as subfolders,
for example

    GUROBIDIR = /opt/gurobi650/linux

and make shure that `$GUROBIDIR/lib` is in the `LD_LIBRARY_PATH` environment variable
so that `libgurobi*.so` can be found.

## METIS
METIS is a requirement of MA57, which only works with METIS versions < 5.
pysleqp Makefile will automatically download and build METIS for you.
Please acknowledge METIS and its licence at [Karypis Lab](http://glaros.dtc.umn.edu/gkhome/metis/metis/overview).

## HSL MA57
Get HSL MA57 from the [HSL MA57 webpage](http://www.hsl.rl.ac.uk/catalogue/hsl_ma57.html)
Make sure to obtain HSL MA57 (Fortran 90) and not MA57 (Fortran 77).
Either unpack the HSL MA57 archive to `ThirdParty/hsl_ma57` or copy
the following files from the HSL MA57 archive to `ThirdParty/hsl_ma57/src`:

* `ddeps90.f90`
* `ddeps.f`
* `hsl_ma57d.f90`

## GLTR
Get GALAHAD at the [GALAHAD webpage](http://www.galahad.rl.ac.uk).
Either unpack the GALAHAD archive to `ThirdParty/galahad` or copy
the following files from the GALAHAD archive to `ThirdParty/galahad`,
make sure to preserve relative paths:

* `seds/double.sed`
* `seds/ma57v4.sed`
* `src/auxiliary/norms.f90`
* `src/gltr/gltr.f90`
* `src/lapack/blas_interface.f90`
* `src/lapack/lapack_interface.f90`
* `src/rand/rand.f90`
* `src/roots/roots.f90`
* `src/sort/sort.f90`
* `src/space/space.f90`
* `src/spec/specfile.f90`
* `src/sym/symbols.f90`

# Compilation
If you have installed all the dependencies, executing `make` in the pysleqp
directory should compile everything into the directory `build`. If you prefer
a target directory different from `build`, edit the pysleqp `Makefile` and
modify the `BUILDDIR` variable.
Make sure that this target directory (`build` per default) is in your
`$PYTHONPATH` environment variable and the gurobi library is in `$LD_LIBRARY_PATH`
as described above.

# Running an example
If everything compiled as expected, you can test the installation by running:

    python example/hs071.py

It should take about 7 iterations and terminate with an objective function value
of about 12.1464.

You can use this file as a starting point to formulate your own NLP.
