! This software is licensed according to the pySLEQP academic license
! see LICENSE

module gltr_wrapper

use iso_c_binding, only: c_double, c_int
use GALAHAD_GLTR_double

implicit none

contains

subroutine c_gltr(n, radius, f, X, R, VECTOR, initialization, num_iter, info, negative_curvature, rayleigh, multiplier) bind(c)
integer(c_int), intent(in) :: n, initialization
integer(c_int), intent(inout) :: num_iter, info
integer(c_int), intent(inout) :: negative_curvature
real(c_double), intent(inout) :: f, radius, rayleigh, multiplier
real(c_double), dimension(n), intent(inout) :: X, R, VECTOR
type(GLTR_data_type), save :: data
type(GLTR_control_type), save :: control
type(GLTR_info_type), save :: inform

if(initialization /= 0) then
    call GLTR_initialize( data, control, inform)
    control%unitm = .FALSE.
    inform%status = 1
    control%itmax = 100*n ! to bypass the GLTR -18 case
    control%extra_vectors = min(100, n)
    return
end if

do
    call GLTR_solve( n, radius, f, X, R, VECTOR, data, control, inform)
    info = inform%status
    select case( inform%status )
    case(2, 6) ! return to projection
        return
    case(3, 7) ! return to computation H*v
        return
    case(5) ! restart
        return
    case(-30, 0) ! succesful termination
        num_iter = inform%iter + inform%iter_pass2
        if (inform%negative_curvature) then
            negative_curvature = 1
        end if
        rayleigh = inform%rayleigh
        multiplier = inform%multiplier
        call GLTR_terminate(data, control, inform)
        return
    case default ! error
        num_iter = inform%iter + inform%iter_pass2
        if (inform%negative_curvature) then
            negative_curvature = 1
        end if
        call GLTR_terminate(data, control, inform)
        return
    end select
end do

end subroutine c_gltr

end module gltr_wrapper
