# This software is licensed according to the pySLEQP academic license
# see LICENSE

import numpy as np
cimport numpy as np

cdef extern:
    void c_ma57(int *n, int *m, int *ne_A, double *val_A, long *row_A, long *col_A, double *vector, int *initialization, int *finalization, int* info, double *precond)

def factor_saddlepoint(A, double precond):
    cdef int initialization = 1
    cdef int finalization = 0
    cdef int info
    cdef int m = A.shape[0]
    cdef int n = A.shape[1]
    cdef int nnz = A.nnz
    cdef long [:] row = np.ascontiguousarray(A.row, dtype=np.long)
    cdef long [:] col = np.ascontiguousarray(A.col, dtype=np.long)
    cdef double [:] data = np.ascontiguousarray(A.data, dtype=np.double)
    if m > 0:
        c_ma57(&n, &m, &nnz, &data[0], &row[0], &col[0], NULL, &initialization, &finalization, &info, &precond)
    else:
        c_ma57(&n, &m, &nnz, NULL, NULL, NULL, NULL, &initialization, &finalization, &info, &precond)
    return info

def finalize_saddlepoint():
    cdef int initialization = 0
    cdef int finalization = 1
    cdef int info
    c_ma57(&initialization, &initialization, &initialization, NULL, NULL, NULL, NULL, &initialization, &finalization, &info, NULL)
    return info

def solve_saddlepoint_overwrite_s_zero_t(int n, int m, double [:] s):
    cdef int initialization = 0
    cdef int finalization = 0
    cdef int info
    cdef double [:] rhs = np.empty(n+m)
    rhs[:n] = s[:]
    rhs[n:] = 0
    c_ma57(&n, &m, &initialization, NULL, NULL, NULL, &rhs[0], &initialization, &finalization, &info, NULL)
    s[:] = rhs[:n]
    return info

def solve_saddlepoint_get_s_zero_s(int n, int m, double [:] t):
    cdef int initialization = 0
    cdef int finalization = 0
    cdef int info
    rhs = np.empty(n+m)
    cdef double [:] rhs_view = rhs
    rhs_view[:n] = 0
    rhs_view[n:] = t[:]
    c_ma57(&n, &m, &initialization, NULL, NULL, NULL, &rhs_view[0], &initialization, &finalization, &info, NULL)
    return rhs[:n]

def solve_saddlepoint_get_t_zero_t(int n, int m, double [:] s):
    cdef int initialization = 0
    cdef int finalization = 0
    cdef int info
    rhs = np.empty(n+m)
    cdef double [:] rhs_view = rhs
    rhs_view[:n] = s[:]
    rhs_view[n:] = 0
    c_ma57(&n, &m, &initialization, NULL, NULL, NULL, &rhs_view[0], &initialization, &finalization, &info, NULL)
    return rhs[n:]
